from motherhamster.imageassets import *

########################################################################
class AnimationAsset(AssetBase):
    """a python object representation of an animation sequence
       loaded from a set of grouped on-disk image files
       it can produce pygame surfaces for use in rendering"""
    
    hotspotx_type = AssetEnum(ImageAsset.x_origins, ImageAsset.x_origins.center)
    hotspoty_type = AssetEnum(ImageAsset.y_origins, ImageAsset.y_origins.center)
    hotspot_offset = AssetVec2d()
    blend_mode = AssetEnum(ImageAsset.blend_modes, ImageAsset.blend_modes.alpha_blend)
    joints = AssetList(AssetVec2d())
    
    playback_rate = AssetFloat(30.0, "frames per sec.", 1.0, 120.0)
    
    def __init__(self):
        self._images = []
    
    def _set_path(self, path):
        dish = GetFoodDish(path.root)
        dir_path = path.filepath
        file_list = []
        extension_list = [".png", ".jpg"]
        dish._RecursiveFileExtensionSearch(file_list, dir_path, extension_list)
        for file_path_obj in file_list:
            image = ImageAsset()
            image._set_file(dish.open(file_path_obj.filepath, "rb"))
            self._images.append(image)
        self._set_image_properties()
            
    def _set_image_properties(self):
        for image in self._images:
            property_dict = image.asset_get_properties()
            for key in property_dict:
                if key in self.__dict__:
                    setattr(image, key, getattr(self, key))
                     
    def asset_set_property(self, key, value):
        AssetBase.asset_set_property(self, key, value)
        if hasattr(ImageAsset, key):
            for image in self._images:
                image.asset_set_property(key, value)
    
    def __len__(self):
        return len(self._images)
    def __getitem__(self, key):
        return self._images[key]

    def get_length_seconds(self):
        """The expected play length in seconds"""
        return len(self._images)/self.playback_rate
    def get_image_at_time(self, seconds):
        """Get the expected frame for the animation"""
        frame_index = int(math.floor(seconds*self.playback_rate)) % len(self._images)
        return self._images[frame_index]
        
    def asset_is_writable(self):
        """Returns whether you will be able to write the asset to disk"""
        try:
            path = self.asset_metadata['path']
        except:
            return False
        metadata_filename = AnimationAsset.get_metadata_path(path)
        writable = GetFoodDish(path.root).is_writable(metadata_filename)
        return writable

    def _get_metadata_path(asset_path):
        return asset_path.filepath + "/metadata"
    get_metadata_path = staticmethod(_get_metadata_path)
            
########################################################################
class _AnimationPouch(GenericAssetPouch):
    """The list and loader for animations"""

    def __init__(self):
        GenericAssetPouch.__init__(self, AnimationAsset)
        self._asset_paths = None

    def AssetExtensions(self):
        return [".ani"]

    def _load(self, asset_path):
        """loads an asset using pickle for the data part and also loads some image part..."""
        path = self.GetAssetPathForID(asset_path)
        if (path):
            dish = GetFoodDish(path.root)
            metadata_filename = AnimationAsset.get_metadata_path(path)
            try:
                metadata_file = dish.open(metadata_filename, 'rb')
            except:
                metadata_file = None
            if (metadata_file):
                asset = pickle.load(metadata_file)
            else:
                asset = AnimationAsset()
            asset._set_path(path)
            return asset
        return None

    def _save(self, asset_path, obj):
        """saves an asset using pickle (just the data part)"""
        path = self.GetAssetPathForID(asset_path)
        if (path):
            dish = GetFoodDish(path.root)
            metadata_filename = AnimationAsset.get_metadata_path(path)
            metadata_file = dish.open(metadata_filename, 'wb')
            pickle.dump(obj, metadata_file)

########################################################################

pouch = _AnimationPouch()
RegisterPouch(pouch)
  